@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 #---
-# Copyright 2003-2011 by Jim Weirich (jim@weirichhouse.org).
+# Copyright 2003-2013 by Jim Weirich (jim.weirich@gmail.com).
 # All rights reserved.
 
 # Permission is granted for use, copying, modification, distribution,
@@ -13,7 +13,6 @@ require 'flexmock/argument_matchers'
 
 class FlexMock
 
-  ####################################################################
   # Include this module in your test class if you wish to use the +eq+
   # and +any+ argument matching methods without a prefix.  (Otherwise
   # use <tt>FlexMock.any</tt> and <tt>FlexMock.eq(obj)</tt>.
@@ -48,6 +47,10 @@ class FlexMock
     def ducktype(*methods)
       DuckMatcher.new(methods)
     end
+
+    def optional_proc
+      OPTIONAL_PROC_MATCHER
+    end
   end
   extend ArgumentTypes
 
