@@ -1,7 +1,7 @@
 #!/usr/bin/env ruby
 
 #---
-# Copyright 2003-2011 by Jim Weirich (jim@weirichhouse.org).
+# Copyright 2003-2013 by Jim Weirich (jim.weirich@gmail.com).
 # All rights reserved.
 
 # Permission is granted for use, copying, modification, distribution,
@@ -9,7 +9,7 @@
 # above copyright notice is included.
 #+++
 
-require 'test/test_setup'
+require 'test_helper'
 
 def mock_top_level_function
   :mtlf
@@ -29,14 +29,14 @@ class Cat
   end
 end
 
-class TestFlexMockShoulds < Test::Unit::TestCase
-  include FlexMock::TestCase
+class TestFlexMockShoulds < Minitest::Test
+  include FlexMock::Minitest
 
   # Expected error messages on failures
   COUNT_ERROR_MESSAGE = /\bcalled\s+incorrect\s+number\s+of\s+times\b/
   NO_MATCH_ERROR_MESSAGE = /\bno\s+matching\s+handler\b/
-  AT_LEAST_ERROR_MESSAGE = /\bshould\s+be\s+called\s+at\s+least\b/
-  AT_MOST_ERROR_MESSAGE = /\bshould\s+be\s+called\s+at\s+most\b/
+  AT_LEAST_ERROR_MESSAGE = COUNT_ERROR_MESSAGE
+  AT_MOST_ERROR_MESSAGE = COUNT_ERROR_MESSAGE
   OUT_OF_ORDER_ERROR_MESSAGE = /\bcalled\s+out\s+of\s+order\b/
   NON_CONTAINER_MESSAGE = /\bis\s+not\s+in\s+a\s+container\b/
 
@@ -149,10 +149,23 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     end
   end
 
+  def test_multiple_yields_and_multiple_returns_are_synced
+    FlexMock.use do |m|
+      m.should_receive(:msg).and_yield(:one).and_return(1).and_yield(:two).and_return(2)
+      yielded_values = []
+      returned_values = []
+      returned_values << m.msg { |a| yielded_values << a }
+      returned_values << m.msg { |a| yielded_values << a }
+      returned_values << m.msg { |a| yielded_values << a }
+      assert_equal [:one, :two, :two], yielded_values
+      assert_equal [1, 2, 2], returned_values
+    end
+  end
+
   def test_failure_if_no_block_given
     FlexMock.use do |m|
       m.should_receive(:hi).and_yield(:one, :two).once
-      assert_raise(FlexMock::MockError) do m.hi end
+      assert_raises(FlexMock::MockError) do m.hi end
     end
   end
 
@@ -208,7 +221,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_and_raises_with_exception_class_throws_exception
     FlexMock.use do |m|
       m.should_receive(:failure).and_raise(MyError)
-      assert_raise MyError do
+      assert_raises MyError do
         m.failure
       end
     end
@@ -217,7 +230,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_and_raises_with_arguments_throws_exception_made_with_args
     FlexMock.use do |m|
       m.should_receive(:failure).and_raise(MyError, "my message")
-      ex = assert_raise MyError do
+      ex = assert_raises MyError do
         m.failure
       end
       assert_equal "my message", ex.message
@@ -228,7 +241,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     FlexMock.use do |m|
       err = MyError.new
       m.should_receive(:failure).and_raise(err)
-      ex = assert_raise MyError do
+      ex = assert_raises MyError do
         m.failure
       end
       assert_equal err, ex
@@ -238,7 +251,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_raises_is_an_alias_for_and_raise
     FlexMock.use do |m|
       m.should_receive(:failure).raises(RuntimeError)
-      assert_raise RuntimeError do
+      assert_raises RuntimeError do
         m.failure
       end
     end
@@ -249,9 +262,9 @@ class TestFlexMockShoulds < Test::Unit::TestCase
       m.should_receive(:failure).
         and_raise(RuntimeError, "ONE").
         and_raise(RuntimeError, "TWO")
-      ex = assert_raise RuntimeError do m.failure end
+      ex = assert_raises RuntimeError do m.failure end
       assert_equal "ONE", ex.message
-      ex = assert_raise RuntimeError do m.failure end
+      ex = assert_raises RuntimeError do m.failure end
       assert_equal "TWO", ex.message
     end
   end
@@ -301,6 +314,13 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     end
   end
 
+  def test_pass_thru_just_returns_undefined_on_mocks
+    FlexMock.use do |m|
+      m.should_receive(:hi).pass_thru
+      assert_equal FlexMock.undefined, m.hi
+    end
+  end
+
   def test_multiple_expectations
     FlexMock.use do |m|
       m.should_receive(:hi).with(1).returns(10)
@@ -319,7 +339,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_with_no_args_but_with_args
-    assert_failure(NO_MATCH_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>NO_MATCH_ERROR_MESSAGE, :deep => true) do
       FlexMock.use do |m|
         m.should_receive(:hi).with_no_args
         m.hi(1)
@@ -372,7 +392,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_with_ducktype_arg_matching_no_match
     FlexMock.use('greeter') do |m|
       m.should_receive(:hi).with(FlexMock.ducktype(:purr, :meow, :growl))
-      assert_failure {
+      assert_mock_failure(check_failed_error, :deep => true, :line => __LINE__+1) {
         m.hi(Cat.new)
       }
     end
@@ -388,7 +408,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_with_hash_non_matching
     FlexMock.use('greeter') do |m|
       m.should_receive(:hi).with(FlexMock.hsh(:a => 1, :b => 2))
-      assert_failure {
+      assert_mock_failure(check_failed_error, :deep => true, :line => __LINE__+1) {
         m.hi(:a => 1, :b => 4, :c => 3)
       }
     end
@@ -403,9 +423,31 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     end
   end
 
+  def test_with_optional_proc
+    FlexMock.use('greeter') do |m|
+      m.should_receive(:hi).with(optional_proc).once
+      m.hi { }
+    end
+  end
+
+  def test_with_optional_proc_and_missing_proc
+    FlexMock.use('greeter') do |m|
+      m.should_receive(:hi).with(optional_proc).once
+      m.hi
+    end
+  end
+
+  def test_with_optional_proc_distinquishes_between_nil_and_missing
+    FlexMock.use('greeter') do |m|
+      m.should_receive(:hi).with(optional_proc).never
+      m.should_receive(:hi).with(nil).once
+      m.hi(nil)
+    end
+  end
+
   def test_with_arbitrary_arg_matching
     FlexMock.use('greeter') do |m|
-      m.should_receive(:hi).with(FlexMock.on { |arg| arg % 2 == 0 }).twice
+      m.should_receive(:hi).with(FlexMock.on { |arg| arg % 2 == 0 rescue nil }).twice
       m.should_receive(:hi).never
       m.should_receive(:hi).with(1).once
       m.should_receive(:hi).with(2).never
@@ -451,8 +493,8 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_arg_matching_with_no_match
     FlexMock.use do |m|
       m.should_receive(:hi).with(1).returns(10)
-      assert_failure(NO_MATCH_ERROR_MESSAGE) {
-        assert_equal 20, m.hi(2)
+      assert_mock_failure(check_failed_error, :message =>NO_MATCH_ERROR_MESSAGE, :deep => true, :line => __LINE__+1) {
+        m.hi(2)
       }
     end
   end
@@ -460,7 +502,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_arg_matching_with_string_doesnt_over_match
     FlexMock.use do |m|
       m.should_receive(:hi).with(String).returns(20)
-      assert_failure(NO_MATCH_ERROR_MESSAGE) {
+      assert_mock_failure(check_failed_error, :message =>NO_MATCH_ERROR_MESSAGE, :deep => true, :line => __LINE__+1) {
         m.hi(1.0)
       }
     end
@@ -469,7 +511,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_block_arg_given_to_no_args
     FlexMock.use do |m|
       m.should_receive(:hi).with_no_args.returns(20)
-      assert_failure(NO_MATCH_ERROR_MESSAGE) {
+      assert_mock_failure(check_failed_error, :message =>NO_MATCH_ERROR_MESSAGE, :deep => true, :line => __LINE__+1) {
         m.hi { 1 }
       }
     end
@@ -509,7 +551,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_never_and_called_once
-    assert_failure(COUNT_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>COUNT_ERROR_MESSAGE, :deep => true, :line => __LINE__+3) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).never
         m.hi(1)
@@ -525,7 +567,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_once_but_never_called
-    assert_failure(COUNT_ERROR_MESSAGE) do
+    assert_mock_failure(assertion_failed_error, :message =>COUNT_ERROR_MESSAGE, :line => __LINE__+2) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).returns(10).once
       end
@@ -533,7 +575,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_once_but_called_twice
-    assert_failure(COUNT_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>COUNT_ERROR_MESSAGE, :line => __LINE__+4) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).returns(10).once
         m.hi(1)
@@ -585,7 +627,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_at_least_but_never_called
-    assert_failure(AT_LEAST_ERROR_MESSAGE) do
+    assert_mock_failure(assertion_failed_error, :message =>AT_LEAST_ERROR_MESSAGE, :line => __LINE__+2) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).returns(10).at_least.once
       end
@@ -601,7 +643,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_at_least_and_exact
-    assert_failure(COUNT_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>COUNT_ERROR_MESSAGE, :line => __LINE__+4) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).returns(10).at_least.once.once
         m.hi(1)
@@ -624,21 +666,23 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_at_most_called_twice
-    assert_failure(AT_MOST_ERROR_MESSAGE) do
+    ex = assert_mock_failure(check_failed_error, :message =>AT_MOST_ERROR_MESSAGE, :line => __LINE__+4) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).returns(10).at_most.once
         m.hi(1)
         m.hi(1)
       end
     end
+    assert_match(/at most 1/i, ex.message)
   end
 
   def test_at_most_and_at_least_called_never
-    assert_failure(AT_LEAST_ERROR_MESSAGE) do
+    ex = assert_mock_failure(assertion_failed_error, :message =>AT_LEAST_ERROR_MESSAGE, :line => __LINE__+2) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).returns(10).at_least.once.at_most.twice
       end
     end
+    assert_match(/at least 1/i, ex.message)
   end
 
   def test_at_most_and_at_least_called_once
@@ -657,7 +701,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_at_most_and_at_least_called_three_times
-    assert_failure(AT_MOST_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>AT_MOST_ERROR_MESSAGE, :line => __LINE__+5) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).returns(10).at_least.once.at_most.twice
         m.hi(1)
@@ -680,7 +724,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_call_counts_only_apply_to_matching_args_with_mismatch
-    ex = assert_failure(COUNT_ERROR_MESSAGE) do
+    ex = assert_mock_failure(assertion_failed_error, :message =>COUNT_ERROR_MESSAGE, :line => __LINE__+3) do
       FlexMock.use do |m|
         m.should_receive(:hi).with(1).once
         m.should_receive(:hi).with(2).twice
@@ -706,7 +750,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_ordered_calls_out_of_order_will_fail
-    assert_failure(OUT_OF_ORDER_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
       FlexMock.use 'm' do |m|
         m.should_receive(:hi).ordered
         m.should_receive(:lo).ordered
@@ -717,6 +761,18 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     end
   end
 
+  def test_failure_in_ordered_calls_combined_with_valid_count_will_report_an_order_failure
+    assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
+      FlexMock.use 'm', 'n' do |m, n|
+        m.should_receive(:hi).once.globally.ordered
+        n.should_receive(:lo).once.globally.ordered
+
+        n.lo
+        m.hi
+      end
+    end
+  end
+
   def test_order_calls_with_different_arg_lists_and_in_order_will_pass
     FlexMock.use 'm' do |m|
       m.should_receive(:hi).with("one").ordered
@@ -728,7 +784,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_order_calls_with_different_arg_lists_and_out_of_order_will_fail
-    assert_failure(OUT_OF_ORDER_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
       FlexMock.use 'm' do |m|
         m.should_receive(:hi).with("one").ordered
         m.should_receive(:hi).with("two").ordered
@@ -795,6 +851,36 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     end
   end
 
+  def test_global_ordering_message_includes_received_calls
+    e = assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
+      FlexMock.use 'm' do |m|
+        one = m.should_receive(:one).globally.ordered
+        two = m.should_receive(:two).globally.ordered
+        m.one
+        m.two
+        m.one
+      end
+    end
+    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
+    assert_match /two\(\) matched by should_receive\(:two\)/, e.message
+    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
+  end
+
+  def test_ordering_message_includes_received_calls
+    e = assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
+      FlexMock.use 'm' do |m|
+        one = m.should_receive(:one).ordered
+        two = m.should_receive(:two).ordered
+        m.one
+        m.two
+        m.one
+      end
+    end
+    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
+    assert_match /two\(\) matched by should_receive\(:two\)/, e.message
+    assert_match /one\(\) matched by should_receive\(:one\)/, e.message
+  end
+
   def test_explicit_ordering_mixed_with_implicit_ordering_should_not_overlap
     FlexMock.use 'm' do |m|
       xstart = m.should_receive(:start).ordered
@@ -806,7 +892,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_explicit_ordering_with_explicit_misorders
-    assert_failure(OUT_OF_ORDER_ERROR_MESSAGE) do
+    assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
       FlexMock.use 'm' do |m|
         m.should_receive(:hi).ordered(:first_group)
         m.should_receive(:lo).ordered(:second_group)
@@ -849,20 +935,30 @@ class TestFlexMockShoulds < Test::Unit::TestCase
       x.should_receive(:one).ordered
       y.should_receive(:two).ordered
 
-      assert_nothing_raised do
+      y.two
+      x.one
+    end
+  end
+
+  def test_ordering_between_mocks_is_honored_for_global_ordering
+    assert_mock_failure(check_failed_error, :message =>OUT_OF_ORDER_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
+      FlexMock.use("x", "y") do |x, y|
+        x.should_receive(:one).globally.ordered
+        y.should_receive(:two).globally.ordered
+
         y.two
         x.one
       end
     end
   end
 
-  def test_ordering_between_mocks_is_honored_for_global_ordering
-    assert_failure(OUT_OF_ORDER_ERROR_MESSAGE) do
+  def test_ordering_is_verified_after_eligibility
+    assert_mock_failure(check_failed_error, :message =>COUNT_ERROR_MESSAGE, :deep => true, :line => __LINE__+6) do
       FlexMock.use("x", "y") do |x, y|
-        x.should_receive(:one).globally.ordered
-        y.should_receive(:two).globally.ordered
-
-        y.two
+        x.should_receive(:one).once.ordered
+        x.should_receive(:two).ordered
+        x.one
+        x.two
         x.one
       end
     end
@@ -939,7 +1035,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   def test_expectations_with_count_constraints_can_by_marked_as_default
     m = flexmock("m")
     m.should_receive(:foo).and_return(:bar).once.by_default
-    assert_raise assertion_failed_error do
+    assert_raises assertion_failed_error do
       flexmock_teardown
     end
   end
@@ -956,7 +1052,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     m = flexmock("m")
     m.should_receive(:foo).with(1).and_return(:bar).once.by_default
     m.should_receive(:foo).with(2).and_return(:baz).once
-    assert_raise assertion_failed_error do
+    assert_raises check_failed_error do
       # This expectation should be hidded by the non-result
       m.foo(1)
     end
@@ -968,7 +1064,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     m.should_receive(:foo).ordered.by_default
     m.should_receive(:bar).ordered.by_default
     m.bar
-    assert_raise assertion_failed_error do m.foo end
+    assert_raises check_failed_error do m.foo end
   end
 
   def test_ordered_default_expectations_can_be_overridden
@@ -995,9 +1091,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
   end
 
   def test_by_default_at_mock_level_does_nothing_with_no_expectations
-    assert_nothing_raised do
-      flexmock("m").by_default
-    end
+    flexmock("m").by_default
   end
 
   def test_partial_mocks_can_have_default_expectations
@@ -1017,7 +1111,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     mock = flexmock("mock")
     exp = mock.should_receive(:foo).and_return(:first).once
     mock.should_receive(:foo).and_return(:second)
-    ex = assert_raise(FlexMock::UsageError) do
+    ex = assert_raises(FlexMock::UsageError) do
       exp.by_default
     end
     assert_match %r(previously defined), ex.message
@@ -1036,6 +1130,20 @@ class TestFlexMockShoulds < Test::Unit::TestCase
     assert ! mock.respond_to?(:phoo, true)
   end
 
+  def test_backtraces_point_to_should_receive_line
+    mock = flexmock("a mock")
+    file_name_re = Regexp.quote(__FILE__)
+    line_no = __LINE__ + 1
+    mock.should_receive(:foo).and_return(:bar).once
+    begin
+      flexmock_verify
+    rescue Exception => ex
+      exception = ex
+    end
+    refute_nil exception
+    assert_match(/#{file_name_re}:#{line_no}/, exception.backtrace.first)
+  end
+
   def test_can_mock_operators
     assert_operator(:[]) { |m| m[1] }
     assert_operator(:[]=) { |m| m[1] = :value }
@@ -1074,7 +1182,7 @@ class TestFlexMockShoulds < Test::Unit::TestCase
 
 end
 
-class TestFlexMockShouldsWithInclude < Test::Unit::TestCase
+class TestFlexMockShouldsWithInclude < Minitest::Test
   include FlexMock::ArgumentTypes
   def test_include_enables_unqualified_arg_type_references
     FlexMock.use("x") do |m|
@@ -1084,9 +1192,9 @@ class TestFlexMockShouldsWithInclude < Test::Unit::TestCase
   end
 end
 
-class TestFlexMockArgTypesDontLeak < Test::Unit::TestCase
+class TestFlexMockArgTypesDontLeak < Minitest::Test
   def test_unqualified_arg_type_references_are_undefined_by_default
-    ex = assert_raise(NameError) do
+    ex = assert_raises(NameError) do
       FlexMock.use("x") do |m|
         m.should_receive(:hi).with(any).once
         m.hi(1)
